use utils::{error::{Error, Result}, is_human};

use actix_web::{get, http::header, App, HttpRequest, HttpResponse, HttpServer, Responder};
use clap::Parser;

pub const TATOEBA_API: &str = "https://tatoeba.org/en/api_v0/search";

/// A proxy of the Tatoeba API with no CORS restrictions.
#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct Args {
    /// The port at which to run.
    #[arg(short, long, default_value_t=3001)]
    port: u16,
}

#[get("/")]
async fn root(request: HttpRequest) -> Result<impl Responder> {
    let target = format!("{TATOEBA_API}?{}", request.query_string());
    if !is_human(&request) {
        return Err(Error::NotHuman { target });
    }
    let resp = reqwest::get(target).await?;
    Ok(HttpResponse::Ok()
        .append_header(header::ContentType(mime::APPLICATION_JSON))
        .body(resp.text().await?))
}

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    HttpServer::new(|| App::new().service(root))
        .bind(("127.0.0.1", Args::parse().port))?
        .run()
        .await
}
